@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -23,18 +23,17 @@
 # This includes both simple settings (like the screen dimensions) and
 # methods that perform standard tasks, like the say and menu methods.
 
-# This will be deleted by the end of this file.
-import renpy.display
+import collections
 import os
 
 # Can we add more config variables?
 locked = False
 
 # Contains help for config variables.
-help = [ ] #@ReservedAssignment
+help = [ ]  # @ReservedAssignment
 
 # The title of the game window.
-window_title = "A Ren'Py Game"
+window_title = None
 
 # An image file containing the window icon image.
 window_icon = None
@@ -130,7 +129,7 @@ force_archives = False
 mouse = None
 
 # The default sound playback sample rate.
-sound_sample_rate = 44100
+sound_sample_rate = 48000
 
 # The amount of time music is faded out between tracks.
 fade_music = 0.0
@@ -232,17 +231,23 @@ frames = 0
 
 # NOT USED: A text editor that is launched at the location of the current
 # statement.
-editor = None # os.environ.get('RENPY_EDITOR', None)
+editor = None  # os.environ.get('RENPY_EDITOR', None)
 
 # NOT USED: Text editor, with arguments to reload or clobber the file - used,
 # for example, to display traceback.txt.
-editor_transient = None # os.environ.get('RENPY_EDITOR_TRANSIENT', editor)
+editor_transient = None  # os.environ.get('RENPY_EDITOR_TRANSIENT', editor)
 
 # NOT USED: The separator used between files in the text editor.
-editor_file_separator = None # os.environ.get('RENPY_EDITOR_FILE_SEPARATOR', '" "')
+editor_file_separator = None  # os.environ.get('RENPY_EDITOR_FILE_SEPARATOR', '" "')
 
 # Enable developer mode?
-developer = False
+developer = False  # Changed to True or False in the init code.
+
+# The value of developer requested by the creator (True, False, or "auto")
+original_developer = False
+
+# The default value of developer that's used when it's set to auto.
+default_developer = False
 
 # A logfile that logging messages are sent to.
 log = None
@@ -297,13 +302,13 @@ autosave_slots = 10
 autosave_frequency = int(os.environ.get("RENPY_AUTOSAVE_FREQUENCY", "200"))
 
 # The callback that is used by the scene statement.
-scene = renpy.exports.scene
+scene = None
 
 # The callback that is used by the show statement.
-show = renpy.exports.show
+show = None
 
 # The callback that is used by the hide statement.
-hide = renpy.exports.hide
+hide = None
 
 # Should we use cPickle or pickle for load/save?
 use_cpickle = True
@@ -378,7 +383,7 @@ gamedir = None
 basedir = None
 renpy_base = None
 commondir = None
-logdir = None # Where log and error files go.
+logdir = None  # Where log and error files go.
 
 # Should we enable OpenGL mode?
 gl_enable = True
@@ -511,7 +516,7 @@ choice_layer = "screens"
 raw_tracebacks = ("RENPY_RAW_TRACEBACKS" in os.environ)
 
 # A function to process texts which should be spoken
-tts_function = renpy.display.tts.default_tts_function
+tts_function = None
 
 # Channels that stop voice playback.
 tts_voice_channels = [ "voice" ]
@@ -655,17 +660,169 @@ rollback_side_size = .2
 # If dpi_scale is less than this, make it 1.0.
 de_minimus_dpi_scale = 1.0
 
-# How big must dpi_scale be to deal with it on windows?
-if renpy.windows and renpy.windows <= (6, 3):
-    windows_dpi_scale_head = 1.3
-else:
-    windows_dpi_scale_head = 1.0
+# Not used.
+windows_dpi_scale_head = 1.0
 
 # Should rollback_side be enabled?
-enable_rollback_side = False
+enable_rollback_side = True
+
+# The default contents of the replay scope.
+replay_scope = { "_game_menu_screen" : "preferences" }
+
+# The mixer to use for auto-defined movie channels.
+movie_mixer = "music"
+
+# Auto audio channels. A map from base name to:
+# * mixer
+# * file prefix
+# * file suffix
+auto_channels = { "audio" : ( "sfx", "", ""  ) }
+
+# The channel used by renpy.play.
+play_channel = "audio"
+
+# An image attribute that is added when the character is speaking, and
+# removed when the character is not.
+speaking_attribute = None
+
+# How many elements need to be in a list before we compress it for rollback.
+list_compression_length = 25
+
+# How many elements of history are kept. None to disable history.
+history_length = None
+
+# History callbacks that annotate additional information onto the History
+# object.
+history_callbacks = [ ]
+
+# Should we use the new order for translate blocks?
+new_translate_order = True
+
+# Should we defer style execution until translate block time?
+defer_styles = False
+
+# A list of stores that should be cleaned on translate.
+translate_clean_stores = [ ]
+
+# A list of additional script files that should be translated.
+translate_files = [ ]
+
+# A list of files for which ##<space> comment sequences should also be
+# translated.
+translate_comments = [ ]
+
+# Should we pass the full argument list to the say screen?
+old_say_args = False
+
+# The text-to-speech voice.
+tts_voice = None
+
+# The maximum size of xfit, yfit, first_fit, etc.
+max_fit_size = 8192
+
+# Should the window max size be enforced?
+enforce_window_max_size = True
+
+# The max priority to translate to.
+translate_launcher = False
+
+# A map from language to a list of callbacks that are used to help set it
+# up.
+language_callbacks = collections.defaultdict(list)
+
+# A function that is called to init system styles.
+init_system_styles = None
+
+# Callbacks that are called just before rebuilding styles.
+build_styles_callbacks = [ ]
+
+# Should movie displayables be given their own channels?
+auto_movie_channel = True
+
+# Should we ignore duplicate labels?
+ignore_duplicate_labels = False
+
+# A list of callbacks when creating a line log entry.
+line_log_callbacks = [ ]
+
+# A list of screens for which screen profiling should be enabled.
+profile_screens = [ ]
+
+# Should Ren'Py search for system fonts.
+allow_sysfonts = False
+
+# Should Ren'Py tightly loop during fadeouts? (That is, not stop the fadeout
+# if it reaches the end of a trac.)
+tight_loop_default = True
+
+# Should Ren'Py apply style_prefix to viewport scrollbar styles?
+prefix_viewport_scrollbar_styles = True
+
+# These functions are called to determine if Ren'Py needs to redraw the screen.
+needs_redraw_callbacks = [ ]
+
+# Should a hyperlink inherit the size of the text its in?
+hyperlink_inherit_size = True
+
+# A list of callbacks that are called when a line is printed to stdout or
+# stderr.
+stdout_callbacks = [ ]
+stderr_callbacks = [ ]
+
+# Should ATL automatically cause polar motion when angle changes.
+automatic_polar_motion = True
+
+# Functions that are called to generate lint stats.
+lint_stats_callbacks = [ ]
+
+# Should we apply position properties to the side of a viewport?
+position_viewport_side = True
+
+# Things that be given properties via Character.
+character_id_prefixes = [ ]
+
+# Should surfaces be cached?
+cache_surfaces = True
+
+# Should {nw} wait for voice.
+nw_voice = True
+
+# If not None, a function that's used to process say arguments.
+say_arguments_callback = None
+
+# Should we show an atl interpolation for one frame?
+atl_one_frame = True
+
+# Should we keep the show layer state?
+keep_show_layer_state = True
+
+# A list of callbacks that are called when fast skipping happens.
+fast_skipping_callbacks = [ ]
+
+# Should the audio periodic callback run in its own thread.
+audio_periodic_thread = True
+
+# A list of fonts to preload on Ren'Py startup.
+preload_fonts = [ ]
+
+# Should Ren'Py process multiple ATL events in a single update?
+atl_multiple_events = True
 
-del renpy
 del os
+del collections
+
 
 def init():
-    pass
+    import renpy.display
+
+    global scene
+    scene = renpy.exports.scene
+
+    global show
+    show = renpy.exports.show
+
+    global hide
+    hide = renpy.exports.hide
+
+    global tts_function
+    tts_function = renpy.display.tts.default_tts_function
