@@ -1,4 +1,4 @@
-# Copyright 2004-2015 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -22,9 +22,11 @@
 # This file contains code for initializing and managing the display
 # window.
 
+from __future__ import print_function
 import renpy.display
 import renpy.audio
 import renpy.text
+import renpy.test
 
 import pygame_sdl2 as pygame
 
@@ -33,21 +35,22 @@ import os
 import time
 import cStringIO
 import threading
+import copy
 
 import_time = time.time()
 
 try:
-    import android # @UnresolvedImport
+    import android  # @UnresolvedImport
 except:
     android = None
 
 TIMEEVENT = pygame.event.register("TIMEEVENT")
-PERIODIC =  pygame.event.register("PERIODIC")
+PERIODIC = pygame.event.register("PERIODIC")
 REDRAW = pygame.event.register("REDRAW")
 EVENTNAME = pygame.event.register("EVENTNAME")
 
 # All events except for TIMEEVENT and REDRAW
-ALL_EVENTS = set(pygame.event.get_standard_events()) # @UndefinedVariable
+ALL_EVENTS = set(pygame.event.get_standard_events())  # @UndefinedVariable
 ALL_EVENTS.add(PERIODIC)
 ALL_EVENTS.add(EVENTNAME)
 
@@ -104,6 +107,7 @@ PERIODIC_INTERVAL = 50
 time_base = 0.0
 time_mult = 1.0
 
+
 def init_time():
     warp = os.environ.get("RENPY_TIMEWARP", "1.0")
 
@@ -113,10 +117,12 @@ def init_time():
     time_base = time.time()
     time_mult = float(warp)
 
+
 def get_time():
     t = time.time()
     return time_base + (t - time_base) * time_mult
 
+
 def displayable_by_tag(layer, tag):
     """
     Get the displayable on the given layer with the given tag.
@@ -124,6 +130,7 @@ def displayable_by_tag(layer, tag):
 
     return renpy.game.context().scene_lists.get_displayable_by_tag(layer, tag)
 
+
 class IgnoreEvent(Exception):
     """
     Exception that is raised when we want to ignore an event, but
@@ -132,6 +139,7 @@ class IgnoreEvent(Exception):
 
     pass
 
+
 class EndInteraction(Exception):
     """
     Exception that can be raised (for example, during the render method of
@@ -141,6 +149,7 @@ class EndInteraction(Exception):
     def __init__(self, value):
         self.value = value
 
+
 class absolute(float):
     """
     This represents an absolute float coordinate.
@@ -197,6 +206,39 @@ def place(width, height, sw, sh, placement):
 
     return x, y
 
+
+class DisplayableArguments(renpy.object.Object):
+    """
+    Represents a set of arguments that can be passed to a duplicated
+    displayable.
+    """
+
+    # The name of the displayable without any arguments.
+    name = ()
+
+    # Arguments supplied.
+    args = ()
+
+    # This gets set to true if the arguments are consumed.
+    consumed = False
+
+    # The style prefix in play. This is used by DynamicImage to figure
+    # out the prefix list to apply.
+    prefix = None
+
+    def copy(self, **kwargs):
+        """
+        Returns a copy of this object with the various fields set to the
+        values they were given in kwargs.
+        """
+
+        rv = DisplayableArguments()
+        rv.__dict__.update(self.__dict__)
+        rv.__dict__.update(kwargs)
+
+        return rv
+
+
 class Displayable(renpy.object.Object):
     """
     The base class for every object in Ren'Py that can be
@@ -246,11 +288,82 @@ class Displayable(renpy.object.Object):
     # Does this displayable use the scope?
     _uses_scope = False
 
-    def __init__(self, focus=None, default=False, style='default', **properties):
-        self.style = renpy.style.Style(style, properties) # @UndefinedVariable
+    # Arguments supplied to this displayable.
+    _args = DisplayableArguments()
+
+    # Set to true of the displayable is duplicatable (has a non-trivial
+    # duplicate method), or one of its children is.
+    _duplicatable = False
+
+    # Does this displayable require clipping?
+    _clipping = False
+
+    def __init__(self, focus=None, default=False, style='default', _args=None, **properties):
+        self.style = renpy.style.Style(style, properties)  # @UndefinedVariable
         self.focus_name = focus
         self.default = default
 
+        if _args is not None:
+            self._args = _args
+
+    def _copy(self, args=None):
+        """
+        Makes a shallow copy of the displayable. If `args` is provided,
+        replaces the arguments with the stored copy.
+        """
+
+        rv = copy.copy(self)
+
+        if args is not None:
+            rv._args = args
+
+        return rv
+
+    def _duplicate(self, args):
+        """
+        Makes a duplicate copy of the following kids of displayables:
+
+        * Displayables that can accept arguments.
+        * Displayables that maintain state that should be reset before being
+          shown to the user.
+        * Containers that contain (including transitively) one of the other
+          kinds of displayables.
+
+        Displayables that contain state that can be manipulated by the user
+        are never copied.
+
+        This should call _unique on children that have been copied before
+        setting its own _duplicatable flag.
+        """
+
+        return self
+
+    def _in_current_store(self):
+        """
+        Returns a version of this displayable that will not change as it is
+        rendered.
+        """
+
+        return self
+
+    def _unique(self):
+        """
+        This is called when a displayable is "born" unique, which occurs
+        when there is only a single reference to it. What it does is to
+        manage the _duplicatable flag - setting it false unless one of
+        the displayable's children happens to be duplicatable.
+        """
+
+        return
+
+    def parameterize(self, name, parameters):
+        """
+        Obsolete alias for _duplicate.
+        """
+
+        a = self._args.copy(name=name, args=parameters)
+        return self._duplicate(a)
+
     def _equals(self, o):
         """
         This is a utility method that can be called by a Displayable's
@@ -293,7 +406,6 @@ class Displayable(renpy.object.Object):
 
             i.find_focusable(callback, focus_name)
 
-
     def focus(self, default=False):
         """
         Called to indicate that this widget has the focus.
@@ -302,8 +414,7 @@ class Displayable(renpy.object.Object):
         self.set_style_prefix(self.role + "hover_", True)
 
         if not default:
-            if self.style.hover_sound:
-                renpy.audio.music.play(self.style.hover_sound, channel="sound")
+            renpy.exports.play(self.style.hover_sound)
 
     def unfocus(self, default=False):
         """
@@ -334,18 +445,6 @@ class Displayable(renpy.object.Object):
         self.style.set_prefix(prefix)
         renpy.display.render.redraw(self, 0)
 
-    def parameterize(self, name, parameters):
-        """
-        Called to parameterize this. By default, we don't take any
-        parameters.
-        """
-
-        if parameters:
-            raise Exception("Image '%s' can't take parameters '%s'. (Perhaps you got the name wrong?)" %
-                            (' '.join(name), ' '.join(parameters)))
-
-        return self
-
     def render(self, width, height, st, at):
         """
         Called to display this displayable. This is called with width
@@ -494,11 +593,11 @@ class Displayable(renpy.object.Object):
         Called when the displayable is added to a scene list.
         """
 
-    def _get_parameterized(self):
+    def _target(self):
         """
-        If this is a ImageReference to a parameterized image, return
-        the get_parameterized() of the parameterized image. Otherwise,
-        return this displayable.
+        If this displayable is part of a chain of one or more references,
+        returns the ultimate target of those references. Otherwise, returns
+        the displayable.
         """
 
         return self
@@ -519,23 +618,20 @@ class Displayable(renpy.object.Object):
 
         return
 
-    def _in_old_scene(self):
-        """
-        Returns a version of this displayable that will not change as it is
-        rendered.
-        """
-
-        return self
-
     def _tts_common(self, default_alt=None):
 
         rv = [ ]
 
         for i in self.visit():
             if i is not None:
-                rv.append(i._tts())
+                speech = i._tts()
 
-        rv = " ".join(rv)
+                if speech.strip():
+                    rv.append(speech)
+
+        rv = ": ".join(rv)
+        rv = rv.replace("::", ":")
+        rv = rv.replace(": :", ":")
 
         alt = self.style.alt
 
@@ -551,12 +647,11 @@ class Displayable(renpy.object.Object):
         """
         Returns the self-voicing text of this displayable and all of its
         children that cannot take focus. If the displayable can take focus,
-        retuns the empty string.
+        returns the empty string.
         """
 
         return self._tts_common()
 
-
     def _tts_all(self):
         """
         Returns the self-voicing text of this displayable and all of its
@@ -615,7 +710,7 @@ class SceneLists(renpy.object.Object):
     things to the user.
     """
 
-    __version__ = 6
+    __version__ = 7
 
     def after_setstate(self):
         for i in renpy.config.layers + renpy.config.top_layers:
@@ -650,6 +745,9 @@ class SceneLists(renpy.object.Object):
         if version < 6:
             self.shown = self.image_predict_info
 
+        if version < 7:
+            self.layer_transform = { }
+
     def __init__(self, oldsl, shown):
 
         super(SceneLists, self).__init__()
@@ -678,6 +776,10 @@ class SceneLists(renpy.object.Object):
         # drags with names.
         self.drag_group = None
 
+        # A map from a layer to the transform that applies to that
+        # layer.
+        self.layer_transform = { }
+
         if oldsl:
 
             for i in renpy.config.layers + renpy.config.top_layers:
@@ -703,6 +805,8 @@ class SceneLists(renpy.object.Object):
 
             self.drag_group = oldsl.drag_group
 
+            self.layer_transform.update(oldsl.layer_transform)
+
         else:
             for i in renpy.config.layers + renpy.config.top_layers:
                 self.layers[i] = [ ]
@@ -743,7 +847,7 @@ class SceneLists(renpy.object.Object):
 
         return True
 
-    def transform_state(self, old_thing, new_thing):
+    def transform_state(self, old_thing, new_thing, execution=False):
         """
         If the old thing is a transform, then move the state of that transform
         to the new thing.
@@ -756,17 +860,20 @@ class SceneLists(renpy.object.Object):
         if isinstance(new_thing, renpy.display.screen.ScreenDisplayable):
             return new_thing
 
-        old_transform = old_thing._get_parameterized()
+        old_transform = old_thing._target()
         if not isinstance(old_transform, renpy.display.motion.Transform):
             return new_thing
 
-        new_transform = new_thing._get_parameterized()
+        new_transform = new_thing._target()
         if not isinstance(new_transform, renpy.display.motion.Transform):
             new_thing = new_transform = renpy.display.motion.Transform(child=new_thing)
 
         new_transform.take_state(old_transform)
-        return new_thing
 
+        if execution:
+            new_transform.take_execution_state(old_transform)
+
+        return new_thing
 
     def find_index(self, layer, tag, zorder, behind):
         """
@@ -808,7 +915,6 @@ class SceneLists(renpy.object.Object):
 
         return add_index, remove_index, zorder
 
-
     def add(self,
             layer,
             thing,
@@ -887,9 +993,9 @@ class SceneLists(renpy.object.Object):
                 st = sle.show_time
 
             if (not atl and
-                not at_list and
-                renpy.config.keep_running_transform and
-                isinstance(old, renpy.display.motion.Transform)):
+                    not at_list and
+                    renpy.config.keep_running_transform and
+                    isinstance(old, renpy.display.motion.Transform)):
 
                 thing = sle.displayable._change_transform_child(thing)
             else:
@@ -1002,7 +1108,7 @@ class SceneLists(renpy.object.Object):
         if layer not in self.layers:
             raise Exception("Trying to remove something from non-existent layer '%s'." % layer)
 
-        _add_index, remove_index, zorder = self.find_index(layer, thing, 0, [ ])
+        _add_index, remove_index, _zorder = self.find_index(layer, thing, 0, [ ])
 
         if remove_index is not None:
             tag = self.layers[layer][remove_index].tag
@@ -1021,7 +1127,7 @@ class SceneLists(renpy.object.Object):
         totally wiped out.
         """
 
-        if not layer in self.layers:
+        if layer not in self.layers:
             return
 
         if not hide:
@@ -1038,16 +1144,19 @@ class SceneLists(renpy.object.Object):
         self.shown.predict_scene(layer)
         self.layer_at_list[layer] = (None, [ ])
 
-    def set_layer_at_list(self, layer, at_list):
+    def set_layer_at_list(self, layer, at_list, reset=True):
         self.layer_at_list[layer] = (None, list(at_list))
 
+        if reset:
+            self.layer_transform[layer] = None
+
     def set_times(self, time):
         """
         This finds entries with a time of None, and replaces that
         time with the given time.
         """
 
-        for l, (t, list) in self.layer_at_list.items(): #@ReservedAssignment
+        for l, (t, list) in self.layer_at_list.items():  # @ReservedAssignment
             self.layer_at_list[l] = (t or time, list)
 
         for l, ll in self.layers.iteritems():
@@ -1072,24 +1181,33 @@ class SceneLists(renpy.object.Object):
         rv = renpy.display.layout.MultiBox(layout='fixed', focus=layer, **properties)
         rv.append_scene_list(self.layers[layer])
         rv.layer_name = layer
+        rv._duplicatable = False
 
         time, at_list = self.layer_at_list[layer]
 
+        old_transform = self.layer_transform.get(layer, None)
+        new_transform = None
+
         if at_list:
 
             for a in at_list:
 
                 if isinstance(a, renpy.display.motion.Transform):
                     rv = a(child=rv)
+                    new_transform = rv
                 else:
                     rv = a(rv)
 
+            if (new_transform is not None) and (renpy.config.keep_show_layer_state):
+                self.transform_state(old_transform, new_transform, execution=True)
+
             f = renpy.display.layout.MultiBox(layout='fixed')
             f.add(rv, time, time)
             f.layer_name = layer
 
             rv = f
 
+        self.layer_transform[layer] = new_transform
 
         return rv
 
@@ -1135,6 +1253,25 @@ class SceneLists(renpy.object.Object):
 
             self.layers[l] = newl
 
+    def remove_all_hidden(self):
+        """
+        Removes everything hidden, even if it's not time yet. (Used when making a rollback copy).
+        """
+
+        for l in self.layers:
+            newl = [ ]
+
+            for sle in self.layers[l]:
+
+                if sle.tag:
+
+                    if "$" in sle.tag:
+                        continue
+
+                newl.append(sle)
+
+            self.layers[l] = newl
+
     def get_displayable_by_tag(self, layer, tag):
         """
         Returns the displayable on the layer with the given tag, or None
@@ -1252,6 +1389,7 @@ class MouseMove(object):
         renpy.display.draw.set_mouse_pos(x, y)
         return True
 
+
 def get_safe_mode():
     """
     Returns true if we should go into safe mode.
@@ -1396,21 +1534,20 @@ class Interface(object):
             if layer in renpy.config.layer_clipping:
                 x, y, w, h = renpy.config.layer_clipping[layer]
                 self.layer_properties[layer] = dict(
-                    xpos = x,
-                    xanchor = 0,
-                    ypos = y,
-                    yanchor = 0,
-                    xmaximum = w,
-                    ymaximum = h,
-                    xminimum = w,
-                    yminimum = h,
-                    clipping = True,
+                    xpos=x,
+                    xanchor=0,
+                    ypos=y,
+                    yanchor=0,
+                    xmaximum=w,
+                    ymaximum=h,
+                    xminimum=w,
+                    yminimum=h,
+                    clipping=True,
                     )
 
             else:
                 self.layer_properties[layer] = dict()
 
-
         # A stack giving the values of self.transition and self.transition_time
         # for contexts outside the current one. This is used to restore those
         # in the case where nothing has changed in the new context.
@@ -1434,13 +1571,14 @@ class Interface(object):
         # The thread that can do display operations.
         self.thread = threading.current_thread()
 
-        # Initialize pygame.
-        if pygame.version.vernum < (1, 8, 1):
-            raise Exception("Ren'Py requires pygame 1.8.1 to run.")
-
-        # pygame.font.init()
+        # Initialize audio.
         renpy.audio.audio.init()
-        pygame.display.init()
+
+        # Initialize pygame.
+        try:
+            pygame.display.init()
+        except:
+            pass
 
         # Init timing.
         init_time()
@@ -1511,7 +1649,6 @@ class Interface(object):
         # True if this is the first interact.
         self.start_interact = True
 
-
     def setup_dpi_scaling(self):
 
         if "RENPY_HIGHDPI" in os.environ:
@@ -1554,8 +1691,6 @@ class Interface(object):
             renpy.display.log.exception()
             return 1.0
 
-
-
     def start(self):
         """
         Starts the interface, by opening a window and setting the mode.
@@ -1564,6 +1699,8 @@ class Interface(object):
         if self.started:
             return
 
+        renpy.display.render.render_ready()
+
         # Kill off the presplash.
         renpy.display.presplash.end()
 
@@ -1574,7 +1711,7 @@ class Interface(object):
         self.set_mode()
 
         # Load the image fonts.
-        renpy.text.font.load_image_fonts()
+        renpy.text.font.load_fonts()
 
         # Setup periodic event.
         pygame.time.set_timer(PERIODIC, PERIODIC_INTERVAL)
@@ -1590,7 +1727,7 @@ class Interface(object):
         renpy.display.log.write(s)
 
         if renpy.android and not renpy.config.log_to_stdout:
-            print s
+            print(s)
 
     def post_init(self):
         """
@@ -1599,7 +1736,7 @@ class Interface(object):
 
         # Needed for Unity.
         wmclass = renpy.config.save_directory or os.path.basename(sys.argv[0])
-        os.environ['SDL_VIDEO_X11_WMCLASS'] = wmclass
+        os.environ[b'SDL_VIDEO_X11_WMCLASS'] = wmclass.encode("utf-8")
 
         self.set_window_caption(force=True)
         self.set_icon()
@@ -1622,8 +1759,6 @@ class Interface(object):
 
             pygame.event.set_blocked(i)
 
-
-
     def set_icon(self):
         """
         This is called to set up the window icon.
@@ -1632,15 +1767,11 @@ class Interface(object):
         # Window icon.
         icon = renpy.config.window_icon
 
-        if renpy.windows and renpy.config.windows_icon:
-            icon = renpy.config.windows_icon
-
         if icon:
 
             im = renpy.display.scale.image_load_unscaled(
                 renpy.loader.load(icon),
                 icon,
-                convert=False,
                 )
 
             # Convert the aspect ratio to be square.
@@ -1650,14 +1781,16 @@ class Interface(object):
             square_im.blit(im, ( (imax-iw)/2, (imax-ih)/2 ))
             im = square_im
 
-            if renpy.windows and im.get_size() != (32, 32):
-                im = renpy.display.scale.real_smoothscale(im, (32, 32))
-
             pygame.display.set_icon(im)
 
-
     def set_window_caption(self, force=False):
-        caption = renpy.config.window_title + renpy.store._window_subtitle
+
+        window_title = renpy.config.window_title
+
+        if window_title is None:
+            window_title = "A Ren'Py Game"
+
+        caption = renpy.translation.translate_string(window_title) + renpy.store._window_subtitle
 
         if renpy.exports.get_autoreload():
             caption += " - autoreload"
@@ -1732,7 +1865,6 @@ class Interface(object):
 
         return rv
 
-
     def kill_textures(self):
         renpy.display.render.free_memory()
         renpy.text.text.layout_cache_clear()
@@ -1758,8 +1890,8 @@ class Interface(object):
 
         if self.display_reset:
 
-            pygame.key.stop_text_input() # @UndefinedVariable
-            pygame.key.set_text_input_rect(None) # @UndefinedVariable
+            pygame.key.stop_text_input()  # @UndefinedVariable
+            pygame.key.set_text_input_rect(None)  # @UndefinedVariable
             self.text_rect = None
 
             renpy.display.draw.deinit()
@@ -1771,13 +1903,13 @@ class Interface(object):
                 # minimized state on windows.
                 pygame.display.quit()
 
-            renpy.display.render.free_memory()
-            renpy.display.im.cache.clear()
-            renpy.text.text.layout_cache_clear()
+        renpy.display.render.free_memory()
+        renpy.display.im.cache.clear()
+        renpy.text.text.layout_cache_clear()
 
-            renpy.display.module.bo_cache = None
+        renpy.display.module.bo_cache = None
 
-            self.kill_textures_and_surfaces()
+        self.kill_textures_and_surfaces()
 
         self.old_text_rect = None
         self.display_reset = False
@@ -1785,7 +1917,7 @@ class Interface(object):
         virtual_size = (renpy.config.screen_width, renpy.config.screen_height)
 
         if physical_size is None:
-            if renpy.mobile or renpy.game.preferences.physical_size is None: #@UndefinedVariable
+            if renpy.mobile or renpy.game.preferences.physical_size is None:  # @UndefinedVariable
                 physical_size = (None, None)
             else:
                 physical_size = renpy.game.preferences.physical_size
@@ -1840,7 +1972,6 @@ class Interface(object):
         # True if we're doing a one-time profile.
         self.profile_once = False
 
-
     def draw_screen(self, root_widget, fullscreen_video, draw):
 
         surftree = renpy.display.render.render_screen(
@@ -1858,7 +1989,6 @@ class Interface(object):
         self.surftree = surftree
         self.fullscreen_video = fullscreen_video
 
-
     def take_screenshot(self, scale, background=False):
         """
         This takes a screenshot of the current screen, and stores it so
@@ -1898,7 +2028,6 @@ class Interface(object):
         self.screenshot = sio.getvalue()
         sio.close()
 
-
     def check_background_screenshot(self):
         """
         Handles requests for a background screenshot.
@@ -1930,7 +2059,6 @@ class Interface(object):
 
         return rv
 
-
     def lose_screenshot(self):
         """
         This deallocates the saved screenshot.
@@ -1939,7 +2067,6 @@ class Interface(object):
         self.screenshot = None
         self.screenshot_surface = None
 
-
     def save_screenshot(self, filename):
         """
         Saves a full-size screenshot in the given filename.
@@ -1952,12 +2079,12 @@ class Interface(object):
 
         try:
             renpy.display.scale.image_save_unscaled(window, filename)
+            return True
         except:
             if renpy.config.debug:
                 raise
-            pass
-
 
+            return False
 
     def show_window(self):
 
@@ -1971,7 +2098,14 @@ class Interface(object):
             return
 
         if renpy.config.empty_window:
-            renpy.config.empty_window()
+
+            old_history = renpy.store._history  # @UndefinedVariable
+            renpy.store._history = False
+
+            try:
+                renpy.config.empty_window()
+            finally:
+                renpy.store._history = old_history
 
     def do_with(self, trans, paired, clear=False):
 
@@ -2017,7 +2151,6 @@ class Interface(object):
         scene_lists.replace_transient()
         scene_lists.shown_window = False
 
-
     def set_transition(self, transition, layer=None, force=False):
         """
         Sets the transition that will be performed as part of the next
@@ -2032,7 +2165,6 @@ class Interface(object):
         else:
             self.transition[layer] = transition
 
-
     def event_peek(self):
         """
         This peeks the next event. It returns None if no event exists.
@@ -2068,7 +2200,6 @@ class Interface(object):
 
         return rv
 
-
     def event_wait(self):
         """
         This is in its own function so that we can track in the
@@ -2105,7 +2236,6 @@ class Interface(object):
 
         renpy.ui.close()
 
-
     def compute_scene(self, scene_lists):
         """
         This converts scene lists into a dictionary mapping layer
@@ -2127,7 +2257,6 @@ class Interface(object):
 
         return rv
 
-
     def quit_event(self):
         """
         This is called to handle the user invoking a quit.
@@ -2155,15 +2284,14 @@ class Interface(object):
         else:
             raise renpy.game.QuitException()
 
-
     def get_mouse_info(self):
         # Figure out if the mouse visibility algorithm is hiding the mouse.
-        if self.mouse_event_time + renpy.config.mouse_hide_time < renpy.display.core.get_time():
+        if (renpy.config.mouse_hide_time is not None) and (self.mouse_event_time + renpy.config.mouse_hide_time < renpy.display.core.get_time()):
             visible = False
         else:
             visible = renpy.store.mouse_visible and (not renpy.game.less_mouse)
 
-        visible = visible and self.show_mouse
+        visible = visible and self.show_mouse and not (renpy.display.video.fullscreen)
 
         # If not visible, hide the mouse.
         if not visible:
@@ -2243,7 +2371,7 @@ class Interface(object):
 
         renpy.exports.free_memory()
 
-        print "Entered background."
+        print("Entered background.")
 
         while True:
             ev = pygame.event.wait()
@@ -2254,7 +2382,7 @@ class Interface(object):
             if ev.type == pygame.APP_TERMINATING:
                 sys.exit(0)
 
-        print "Entering foreground."
+        print("Entering foreground.")
 
         # Since we came back to life, we can get rid of the
         # auto-reload.
@@ -2285,7 +2413,6 @@ class Interface(object):
 
         renpy.display.log.write("The window was minimized.")
 
-
     def restored(self):
         """
         Called when we are restored from being an icon.
@@ -2333,7 +2460,6 @@ class Interface(object):
         self.ignore_touch = True
         renpy.display.focus.mouse_handler(None, -1, -1, default=False)
 
-
     def text_event_in_queue(self):
         """
         Returns true if the next event in the queue is a text editing event.
@@ -2350,15 +2476,15 @@ class Interface(object):
         Updates the text input state and text rectangle.
         """
 
-        if renpy.store._text_rect is not None: # @UndefinedVariable
-            self.text_rect = renpy.store._text_rect # @UndefinedVariable
+        if renpy.store._text_rect is not None:  # @UndefinedVariable
+            self.text_rect = renpy.store._text_rect  # @UndefinedVariable
 
         if self.text_rect is not None:
 
-            not_shown = pygame.key.has_screen_keyboard_support() and not pygame.key.is_screen_keyboard_shown() # @UndefinedVariable
+            not_shown = pygame.key.has_screen_keyboard_support() and not pygame.key.is_screen_keyboard_shown()  # @UndefinedVariable
 
             if not self.old_text_rect or not_shown:
-                pygame.key.start_text_input() # @UndefinedVariable
+                pygame.key.start_text_input()  # @UndefinedVariable
 
             if self.old_text_rect != self.text_rect:
                 x, y, w, h = self.text_rect
@@ -2366,24 +2492,26 @@ class Interface(object):
                 x1, y1 = renpy.display.draw.untranslate_point(x + w, y + h)
                 rect = (x0, y0, x1 - x0, y1 - y0)
 
-                pygame.key.set_text_input_rect(rect) # @UndefinedVariable
+                pygame.key.set_text_input_rect(rect)  # @UndefinedVariable
 
         else:
             if self.old_text_rect:
-                pygame.key.stop_text_input() # @UndefinedVariable
-                pygame.key.set_text_input_rect(None) # @UndefinedVariable
+                pygame.key.stop_text_input()  # @UndefinedVariable
+                pygame.key.set_text_input_rect(None)  # @UndefinedVariable
 
         self.old_text_rect = self.text_rect
 
-
     def maximum_framerate(self, t):
         """
         Forces Ren'Py to draw the screen at the maximum framerate for `t` seconds.
         """
 
-        self.maximum_framerate_time = max(self.maximum_framerate_time, get_time() + t)
+        if t is None:
+            self.maximum_framerate_time = 0
+        else:
+            self.maximum_framerate_time = max(self.maximum_framerate_time, get_time() + t)
 
-    def interact(self, clear=True, suppress_window=False, **kwargs):
+    def interact(self, clear=True, suppress_window=False, trans_pause=False, **kwargs):
         """
         This handles an interaction, restarting it if necessary. All of the
         keyword arguments are passed off to interact_core.
@@ -2392,6 +2520,8 @@ class Interface(object):
         if not self.started:
             self.start()
 
+        self.trans_pause = trans_pause
+
         # Cancel magic error reporting.
         renpy.bootstrap.report_error = None
 
@@ -2402,7 +2532,6 @@ class Interface(object):
 
         context.interacting = True
 
-
         # Show a missing window.
         if not suppress_window:
             self.show_window()
@@ -2421,7 +2550,7 @@ class Interface(object):
             self.start_interact = True
 
             while repeat:
-                repeat, rv = self.interact_core(preloads=preloads, **kwargs)
+                repeat, rv = self.interact_core(preloads=preloads, trans_pause=trans_pause, **kwargs)
                 self.start_interact = False
 
             return rv
@@ -2441,6 +2570,7 @@ class Interface(object):
 
             self.restart_interaction = True
 
+            renpy.game.context().mark_seen()
             renpy.game.context().scene_lists.shown_window = False
 
     def interact_core(self,
@@ -2452,7 +2582,6 @@ class Interface(object):
                       preloads=[],
                       roll_forward=None,
                       ):
-
         """
         This handles one cycle of displaying an image to the user,
         and then responding to user input.
@@ -2467,6 +2596,13 @@ class Interface(object):
         @param suppress_underlay: This suppresses the display of the underlay.
         """
 
+        suppress_overlay = suppress_overlay or renpy.store.suppress_overlay
+
+        # Store the various parameters.
+        self.suppress_overlay = suppress_overlay
+        self.suppress_underlay = suppress_underlay
+        self.trans_pause = trans_pause
+
         # Show default screens.
         renpy.display.screen.show_overlay_screens(suppress_overlay)
 
@@ -2487,7 +2623,7 @@ class Interface(object):
                 continue
 
             self.ongoing_transition[k] = self.transition[k]
-            self.transition_from[k] = self.old_scene[k]._in_old_scene()
+            self.transition_from[k] = self.old_scene[k]._in_current_store()
             self.transition_time[k] = None
 
         self.transition.clear()
@@ -2497,7 +2633,7 @@ class Interface(object):
             self.transition_from.clear()
             self.transition_time.clear()
 
-        ## Safety condition, prevents deadlocks.
+        # Safety condition, prevents deadlocks.
         if trans_pause:
             if not self.ongoing_transition:
                 return False, None
@@ -2599,7 +2735,7 @@ class Interface(object):
             focus_roots.append(scene_layer)
 
             if (self.ongoing_transition.get(layer, None) and
-                not suppress_transition):
+                    not suppress_transition):
 
                 trans = self.ongoing_transition[layer](
                     old_widget=self.transition_from[layer],
@@ -2623,7 +2759,7 @@ class Interface(object):
 
         # Add layers_root to root_widget, perhaps through a transition.
         if (self.ongoing_transition.get(None, None) and
-            not suppress_transition):
+                not suppress_transition):
 
             old_root = renpy.display.layout.MultiBox(layout='fixed')
             old_root.layers = { }
@@ -2681,12 +2817,19 @@ class Interface(object):
         renpy.display.video.early_interact()
 
         # Call per-interaction code for all widgets.
+        renpy.display.behavior.input_pre_per_interact()
         root_widget.visit_all(lambda i : i.per_interact())
+        renpy.display.behavior.input_post_per_interact()
 
         # Now, update various things regarding scenes and transitions,
         # so we are ready for a new interaction or a restart.
         self.old_scene = scene
 
+        # Something updated the screens. Deal with it now, so the player doesn't
+        # see it.
+        if self.restart_interaction:
+            return True, None
+
         # Okay, from here on we now have a single root widget (root_widget),
         # which we will try to show to the user.
 
@@ -2732,7 +2875,9 @@ class Interface(object):
                 renpy.execution.not_infinite_loop(10)
 
                 # Check for a change in fullscreen preference.
-                if self.fullscreen != renpy.game.preferences.fullscreen or self.display_reset:
+                if ((self.fullscreen != renpy.game.preferences.fullscreen) or
+                        self.display_reset or (renpy.display.draw is None)):
+
                     self.set_mode()
                     needs_redraw = True
 
@@ -2741,6 +2886,10 @@ class Interface(object):
                     renpy.loader.needs_autoreload = False
                     renpy.exports.reload_script()
 
+                for i in renpy.config.needs_redraw_callbacks:
+                    if i():
+                        needs_redraw = True
+
                 # Redraw the screen.
                 if (self.force_redraw or
                     ((first_pass or not pygame.event.peek(ALL_EVENTS)) and
@@ -2762,6 +2911,8 @@ class Interface(object):
                     if not self.interact_time:
                         self.interact_time = self.frame_time
 
+                    renpy.audio.audio.advance_time()  # Sets the time of all video frames.
+
                     self.draw_screen(root_widget, fullscreen_video, (not fullscreen_video) or video_frame_drawn)
 
                     if first_pass:
@@ -2777,25 +2928,25 @@ class Interface(object):
                         new_time = get_time()
 
                         if self.profile_once or (new_time - self.profile_time > .015):
-                            print "Profile: Redraw took %.3f ms." % (1000 * (new_time - self.frame_time))
-                            print "Profile: %.3f ms to complete event." % (1000 * (new_time - self.profile_time))
+                            print("Profile: Redraw took %.3f ms." % (1000 * (new_time - self.frame_time)))
+                            print("Profile: %.3f ms to complete event." % (1000 * (new_time - self.profile_time)))
 
                         self.profile_once = False
 
                     if first_pass and self.last_event and self.last_event.type in [ pygame.MOUSEBUTTONDOWN, pygame.MOUSEBUTTONUP, pygame.MOUSEMOTION ]:
 
-                            x, y = renpy.display.draw.get_mouse_pos()
-                            ev, x, y = renpy.display.emulator.emulator(self.last_event, x, y)
+                        x, y = renpy.display.draw.get_mouse_pos()
+                        ev, x, y = renpy.display.emulator.emulator(self.last_event, x, y)
 
-                            if self.ignore_touch:
-                                x = -1
-                                y = -1
+                        if self.ignore_touch:
+                            x = -1
+                            y = -1
 
-                            if renpy.android and self.last_event.type == pygame.MOUSEBUTTONUP:
-                                x = -1
-                                y = -1
+                        if renpy.android and self.last_event.type == pygame.MOUSEBUTTONUP:
+                            x = -1
+                            y = -1
 
-                            renpy.display.focus.mouse_handler(None, x, y, default=False)
+                        renpy.display.focus.mouse_handler(None, x, y, default=False)
 
                     needs_redraw = False
                     first_pass = False
@@ -2806,6 +2957,8 @@ class Interface(object):
 
                     self.update_text_rect()
 
+                    renpy.test.testexecution.execute()
+
                 # Move the mouse, if necessary.
                 if self.mouse_move is not None:
                     if not self.mouse_move.perform():
@@ -2908,6 +3061,10 @@ class Interface(object):
                     ev = self.event_wait()
 
                 if ev.type == pygame.NOEVENT:
+
+                    if not needs_redraw or self.mouse_move:
+                        pygame.time.wait(1)
+
                     continue
 
                 self.profile_time = get_time()
@@ -3000,13 +3157,12 @@ class Interface(object):
                 # If we're ignoring touch events, and get a mouse up, stop
                 # ignoring those events.
                 if self.ignore_touch and \
-                    ev.type == pygame.MOUSEBUTTONUP and \
-                    ev.button == 1:
+                        ev.type == pygame.MOUSEBUTTONUP and \
+                        ev.button == 1:
 
                     self.ignore_touch = False
                     continue
 
-
                 # Merge mousemotion events.
                 if ev.type == pygame.MOUSEMOTION:
                     evs = pygame.event.get([pygame.MOUSEMOTION])
@@ -3049,6 +3205,7 @@ class Interface(object):
                 # This returns the event location. It also updates the
                 # mouse state as necessary.
                 x, y = renpy.display.draw.mouse_event(ev)
+                x, y = renpy.test.testmouse.get_mouse_pos(x, y)
 
                 ev, x, y = renpy.display.emulator.emulator(ev, x, y)
                 if ev is None:
@@ -3071,7 +3228,7 @@ class Interface(object):
                 try:
 
                     if self.touch:
-                        renpy.display.gesture.recognizer.event(ev, x, y) # @UndefinedVariable
+                        renpy.display.gesture.recognizer.event(ev, x, y)  # @UndefinedVariable
 
                     # Handle the event normally.
                     rv = renpy.display.focus.mouse_handler(ev, x, y)
@@ -3094,7 +3251,6 @@ class Interface(object):
                             l = self.surftree.main_displayables_at_point(x, y, renpy.config.layers)
                             renpy.game.invoke_in_new_context(renpy.config.inspector, l)
 
-
                 except IgnoreEvent:
                     # An ignored event can change the timeout. So we want to
                     # process an TIMEEVENT to ensure that the timeout is
@@ -3103,7 +3259,6 @@ class Interface(object):
                     if ev.type != TIMEEVENT:
                         self.post_time_event()
 
-
                 # Check again after handling the event.
                 needs_redraw |= renpy.display.render.process_redraws()
 
