@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -37,6 +37,7 @@ def scale(num, base):
     else:
         return num
 
+
 class Null(renpy.display.core.Displayable):
     """
     :doc: disp_imagelike
@@ -116,6 +117,36 @@ class Container(renpy.display.core.Displayable):
         rv = self._copy(args)
         rv.children = [ i._duplicate(args) for i in self.children ]
 
+        if rv.children:
+            rv.child = rv.children[-1]
+
+        rv._duplicatable = False
+
+        for i in rv.children:
+            i._unique()
+
+            if i._duplicatable:
+                rv._duplicatable = True
+
+        return rv
+
+    def _in_current_store(self):
+
+        children = [ ]
+
+        changed = False
+
+        for old in self.children:
+            new = old._in_current_store()
+            changed |= (old is not new)
+            children.append(new)
+
+        if not changed:
+            return self
+
+        rv = self._copy()
+        rv.children = children
+
         if rv.children:
             rv.child = rv.children[-1]
 
@@ -155,7 +186,7 @@ class Container(renpy.display.core.Displayable):
         else:
             return
 
-        self.children.pop(i) # W0631
+        self.children.pop(i)  # W0631
         self.offsets = self._list_type()
 
         if self.children:
@@ -163,7 +194,6 @@ class Container(renpy.display.core.Displayable):
         else:
             self.child = None
 
-
     def update(self):
         """
         This should be called if a child is added to this
@@ -172,7 +202,6 @@ class Container(renpy.display.core.Displayable):
 
         renpy.display.render.invalidate(self)
 
-
     def render(self, width, height, st, at):
 
         rv = Render(width, height)
@@ -185,7 +214,6 @@ class Container(renpy.display.core.Displayable):
 
         return rv
 
-
     def event(self, ev, x, y, st):
 
         children = self.children
@@ -224,8 +252,6 @@ class Container(renpy.display.core.Displayable):
         return False
 
 
-
-
 def LiveComposite(size, *args, **properties):
     """
     :doc: disp_imagelike
@@ -265,6 +291,7 @@ def LiveComposite(size, *args, **properties):
 
     return rv
 
+
 class Position(Container):
     """
     Controls the placement of a displayable on the screen, using
@@ -370,8 +397,15 @@ class Grid(Container):
 
     def render(self, width, height, st, at):
 
+        xspacing = self.style.xspacing
+        yspacing = self.style.yspacing
+
+        if xspacing is None:
+            xspacing = self.style.spacing
+        if yspacing is None:
+            yspacing = self.style.spacing
+
         # For convenience and speed.
-        padding = self.style.spacing
         cols = self.cols
         rows = self.rows
 
@@ -396,9 +430,9 @@ class Grid(Container):
         renheight = height
 
         if self.style.xfill:
-            renwidth = (width - (cols - 1) * padding) / cols
+            renwidth = (width - (cols - 1) * xspacing) / cols
         if self.style.yfill:
-            renheight = (height - (rows - 1) * padding) / rows
+            renheight = (height - (rows - 1) * yspacing) / rows
 
         renders = [ render(i, renwidth, renheight, st, at) for i in children ]
         sizes = [ i.get_size() for i in renders ]
@@ -416,8 +450,8 @@ class Grid(Container):
         if self.style.yfill:
             cheight = renheight
 
-        width = cwidth * cols + padding * (cols - 1)
-        height = cheight * rows + padding * (rows - 1)
+        width = cwidth * cols + xspacing * (cols - 1)
+        height = cheight * rows + yspacing * (rows - 1)
 
         rv = renpy.display.render.Render(width, height)
 
@@ -429,8 +463,8 @@ class Grid(Container):
                 child = children[ x + y * cols ]
                 surf = renders[x + y * cols]
 
-                xpos = x * (cwidth + padding)
-                ypos = y * (cheight + padding)
+                xpos = x * (cwidth + xspacing)
+                ypos = y * (cheight + yspacing)
 
                 offset = child.place(rv, xpos, ypos, cwidth, cheight, surf)
                 offsets.append(offset)
@@ -445,6 +479,7 @@ class Grid(Container):
 
         return rv
 
+
 class IgnoreLayers(Exception):
     """
     Raise this to have the event ignored by layers, but reach the
@@ -453,6 +488,7 @@ class IgnoreLayers(Exception):
 
     pass
 
+
 class MultiBox(Container):
 
     layer_name = None
@@ -466,6 +502,8 @@ class MultiBox(Container):
 
         super(MultiBox, self).__init__(style=style, **properties)
 
+        self._clipping = self.style.clipping
+
         self.default_layout = layout
 
         # The start and animation times for children of this
@@ -488,7 +526,7 @@ class MultiBox(Container):
         self.layers = None
         self.scene_list = None
 
-    def _in_old_scene(self):
+    def _in_current_store(self):
 
         if self.layer_name is not None:
 
@@ -502,7 +540,7 @@ class MultiBox(Container):
             for old_sle in self.scene_list:
                 new_sle = old_sle.copy()
 
-                d = new_sle.displayable._in_old_scene()
+                d = new_sle.displayable._in_current_store()
 
                 if d is not new_sle.displayable:
                     new_sle.displayable = d
@@ -525,7 +563,7 @@ class MultiBox(Container):
 
             for layer in renpy.config.layers:
                 old_d = self.layers[layer]
-                new_d = old_d._in_old_scene()
+                new_d = old_d._in_current_store()
 
                 if new_d is not old_d:
                     changed = True
@@ -537,7 +575,7 @@ class MultiBox(Container):
                 return self
 
         else:
-            return self
+            return super(MultiBox, self)._in_current_store()
 
         if self.offsets:
             rv.offsets = list(self.offsets)
@@ -563,7 +601,7 @@ class MultiBox(Container):
         else:
             return "MultiBox"
 
-    def add(self, widget, start_time=None, anim_time=None): # W0221
+    def add(self, widget, start_time=None, anim_time=None):  # W0221
         super(MultiBox, self).add(widget)
         self.start_times.append(start_time)
         self.anim_times.append(anim_time)
@@ -610,12 +648,11 @@ class MultiBox(Container):
             if layout is None:
                 layout = self.default_layout
 
-            self.layout = layout # W0201
+            self.layout = layout  # W0201
 
         else:
             layout = self.layout
 
-
         # Handle time adjustment, store the results in csts and cats.
         if adjust_times:
             t = renpy.game.interface.frame_time
@@ -638,28 +675,41 @@ class MultiBox(Container):
             else:
                 iterator = zip(self.children, csts, cats)
 
-
             rv = renpy.display.render.Render(width, height, layer_name=self.layer_name)
 
-            fit_first = self.style.fit_first
-
             xfit = self.style.xfit
             yfit = self.style.yfit
 
+            fit_first = self.style.fit_first
+
             if fit_first == "width":
-                xfit = True
+                first_fit_width = True
+                first_fit_height = False
             elif fit_first == "height":
-                yfit = True
+                first_fit_width = False
+                first_fit_height = True
             elif fit_first:
-                xfit = True
-                yfit = True
+                first_fit_width = True
+                first_fit_height = True
+            else:
+                first_fit_width = False
+                first_fit_height = False
 
             sizes = [ ]
 
             for child, cst, cat in iterator:
 
                 surf = render(child, width, height, cst, cat)
-                sizes.append(surf.get_size())
+                size = surf.get_size()
+                sizes.append(size)
+
+                if first_fit_width:
+                    width = rv.width = size[0]
+                    first_fit_width = False
+
+                if first_fit_height:
+                    height = rv.height = size[1]
+                    first_fit_height = False
 
                 if surf:
                     offset = child.place(rv, 0, 0, width, height, surf)
@@ -667,7 +717,6 @@ class MultiBox(Container):
                 else:
                     offsets.append((0, 0))
 
-
             if xfit:
                 width = 0
 
@@ -696,7 +745,6 @@ class MultiBox(Container):
                 if height > renpy.config.max_fit_size:
                     raise Exception("Fixed fit width ({}) is too large.".format(height))
 
-
             if self.style.order_reverse:
                 offsets.reverse()
 
@@ -707,7 +755,6 @@ class MultiBox(Container):
         # If we're here, we have a box, either horizontal or vertical. Which is good,
         # as we can share some code between boxes.
 
-
         spacing = self.style.spacing
         first_spacing = self.style.first_spacing
 
@@ -746,7 +793,6 @@ class MultiBox(Container):
         minx = 0
         miny = 0
 
-
         def layout_line(line, xfill, yfill):
             """
             Lays out a single line.
@@ -790,8 +836,6 @@ class MultiBox(Container):
         x = 0
         y = 0
 
-
-
         if layout == "horizontal":
 
             if yfill:
@@ -827,7 +871,6 @@ class MultiBox(Container):
                     remwidth = width
                     line = [ ]
 
-
                 line.append((d, x, y, surf))
                 line_height = max(line_height, sh)
                 x += sw + padding
@@ -835,7 +878,6 @@ class MultiBox(Container):
 
             maxx, maxy = layout_line(line, target_width - x, 0)
 
-
         elif layout == "vertical":
 
             if xfill:
@@ -908,7 +950,6 @@ class MultiBox(Container):
 
         return rv
 
-
     def event(self, ev, x, y, st):
 
         children_offsets = zip(self.children, self.offsets, self.start_times)
@@ -937,9 +978,11 @@ class MultiBox(Container):
 
         return None
 
+
 def Fixed(**properties):
     return MultiBox(layout='fixed', **properties)
 
+
 class SizeGroup(renpy.object.Object):
 
     def __init__(self):
@@ -973,6 +1016,7 @@ class SizeGroup(renpy.object.Object):
 
 size_groups = dict()
 
+
 class Window(Container):
     """
     A window that has padding and margins, and can place a background
@@ -1014,6 +1058,9 @@ class Window(Container):
         xminimum = scale(style.xminimum, width)
         yminimum = scale(style.yminimum, height)
 
+        xmaximum = scale(style.xmaximum, width)
+        ymaximum = scale(style.ymaximum, height)
+
         size_group = self.style.size_group
         if size_group and size_group in size_groups:
             xminimum = max(xminimum, size_groups[size_group].width(width, height, st, at))
@@ -1057,11 +1104,11 @@ class Window(Container):
 
         if renpy.config.enforce_window_max_size:
 
-            if style.xmaximum is not None:
-                width = min(width, style.xmaximum)
+            if xmaximum is not None:
+                width = min(width, xmaximum)
 
-            if style.ymaximum is not None:
-                height = min(height, style.ymaximum)
+            if ymaximum is not None:
+                height = min(height, ymaximum)
 
         rv = renpy.display.render.Render(width, height)
 
@@ -1095,7 +1142,7 @@ class Window(Container):
         if self.child:
             self.offsets = [ offsets ]
 
-        self.window_size = width, height # W0201
+        self.window_size = width, height  # W0201
 
         return rv
 
@@ -1104,6 +1151,7 @@ def dynamic_displayable_compat(st, at, expr):
     child = renpy.python.py_eval(expr)
     return child, None
 
+
 class DynamicDisplayable(renpy.display.core.Displayable):
     """
     :doc: disp_dynamic
@@ -1132,18 +1180,18 @@ class DynamicDisplayable(renpy.display.core.Displayable):
 
     ::
 
-        # If tooltip is not empty, shows it in a text. Otherwise,
-        # show Null. Checks every tenth of a second to see if the
-        # tooltip has been updated.
+        # Shows a countdown from 5 to 0, updating it every tenth of
+        # a second until the time expires.
         init python:
-             def show_tooltip(st, at):
-                 if tooltip:
-                     return tooltip, .1
-                 else:
-                     return Null(), .1
 
-        image tooltipper = DynamicDisplayable(show_tooltip)
+            def show_countdown(st, at):
+                if st > 5.0:
+                    return Text("0.0"), None
+                else:
+                    d = Text("{:.1f}".format(5.0 - st))
+                    return d, 0.1
 
+        image countdown = DynamicDisplayable(show_countdown)
     """
 
     nosave = [ 'child' ]
@@ -1205,16 +1253,18 @@ class DynamicDisplayable(renpy.display.core.Displayable):
 
         return self.child.get_placement()
 
-
     def event(self, ev, x, y, st):
         if self.child:
             return self.child.event(ev, x, y, st)
 
+
 # A cache of compiled conditions used by ConditionSwitch.
 cond_cache = { }
 
 # This chooses the first member of switch that's being shown on the
 # given layer.
+
+
 def condition_switch_pick(switch):
     for cond, d in switch:
 
@@ -1232,9 +1282,11 @@ def condition_switch_pick(switch):
 
     raise Exception("Switch could not choose a displayable.")
 
+
 def condition_switch_show(st, at, switch):
     return condition_switch_pick(switch), None
 
+
 def condition_switch_predict(switch):
 
     if renpy.game.lint:
@@ -1242,6 +1294,7 @@ def condition_switch_predict(switch):
 
     return [ condition_switch_pick(switch) ]
 
+
 def ConditionSwitch(*args, **kwargs):
     """
     :doc: disp_dynamic
@@ -1318,7 +1371,6 @@ def ShowingSwitch(*args, **kwargs):
 
     condargs = [ ]
 
-
     for name, d in zip(args[0::2], args[1::2]):
         if name is not None:
             if not isinstance(name, tuple):
@@ -1327,7 +1379,6 @@ def ShowingSwitch(*args, **kwargs):
         else:
             cond = None
 
-
         condargs.append(cond)
         condargs.append(d)
 
@@ -1368,6 +1419,7 @@ def LiveCrop(rect, child, **properties):
 
     return renpy.display.motion.Transform(child, crop=rect, **properties)
 
+
 class Side(Container):
 
     possible_positions = set([ 'tl', 't', 'tr', 'r', 'br', 'b', 'bl', 'l', 'c'])
@@ -1433,10 +1485,10 @@ class Side(Container):
                 else:
                     return 0, axis
 
-            self.left_space, width = spacer('tl', 'l', 'bl', width) # W0201
-            self.right_space, width = spacer('tr', 'r', 'br', width) # W0201
-            self.top_space, height = spacer('tl', 't', 'tr', height) # W0201
-            self.bottom_space, height = spacer('bl', 'b', 'br', height) # W0201
+            self.left_space, width = spacer('tl', 'l', 'bl', width)  # W0201
+            self.right_space, width = spacer('tr', 'r', 'br', width)  # W0201
+            self.top_space, height = spacer('tl', 't', 'tr', height)  # W0201
+            self.bottom_space, height = spacer('bl', 'b', 'br', height)  # W0201
 
             # The sizes of the various borders.
             left = 0
@@ -1466,13 +1518,13 @@ class Side(Container):
             right, top = sizeit('tr', right, top, right, top)
             right, bottom = sizeit('br', right, bottom, right, bottom)
 
-            self.cwidth = cwidth # W0201
-            self.cheight = cheight # W0201
+            self.cwidth = cwidth  # W0201
+            self.cheight = cheight  # W0201
 
-            self.top = top # W0201
-            self.bottom = bottom # W0201
-            self.left = left # W0201
-            self.right = right # W0201
+            self.top = top  # W0201
+            self.bottom = bottom  # W0201
+            self.left = left  # W0201
+            self.right = right  # W0201
 
         else:
             cwidth = self.cwidth
@@ -1494,7 +1546,6 @@ class Side(Container):
         tops = self.top_space
         bottoms = self.bottom_space
 
-
         if self.style.xfill:
             cwidth = width
 
@@ -1539,7 +1590,9 @@ class Side(Container):
 
         return rv
 
+
 class Alpha(renpy.display.core.Displayable):
+
     def __init__(self, start, end, time, child=None, repeat=False, bounce=False,
                  anim_timebase=False, time_warp=None, **properties):
 
@@ -1654,7 +1707,16 @@ class LiveTile(Container):
 
         for y in range(0, height, ch):
             for x in range(0, width, cw):
-                rv.blit(cr, (x, y), focus=False)
+
+                ccw = min(cw, width - x)
+                cch = min(ch, height - y)
+
+                if (ccw < cw) or (cch < ch):
+                    ccr = cr.subsurface((0, 0, ccw, cch))
+                else:
+                    ccr = cr
+
+                rv.blit(ccr, (x, y), focus=False)
 
         return rv
 
@@ -1753,11 +1815,3 @@ class AlphaMask(Container):
         rv.blit(cr, (0, 0))
 
         return rv
-
-
-
-
-
-
-
-
