@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -232,6 +232,7 @@ class Clipper(object):
 
         return (x0, y0, x1 - x0, y1 - y0), updates
 
+
 clippers = [ Clipper() ]
 
 
@@ -434,7 +435,7 @@ def draw(dest, clip, what, xo, yo, screen):
         return
 
     # Deal with clipping, if necessary.
-    if what.clipping:
+    if what.xclipping or what.yclipping:
 
         if clip:
             cx0, cy0, cx1, cy1 = clip
@@ -567,7 +568,7 @@ def draw_transformed(dest, clip, what, xo, yo, alpha, forward, reverse):
     if what.text_input:
         renpy.display.interface.text_rect = what.screen_rect(xo, yo, reverse)
 
-    if what.clipping:
+    if what.xclipping or what.yclipping:
 
         if reverse.xdy or reverse.ydx:
             draw_transformed(dest, clip, what.pygame_surface(True), xo, yo, alpha, forward, reverse)
@@ -723,6 +724,9 @@ class SWDraw(object):
         # Did we show fullscreen video in the last frame?
         self.showing_video = False
 
+    def get_texture_size(self):
+        return 0, 0
+
     def set_mode(self, virtual_size, physical_size, fullscreen):
 
         # Reset before resize.
