@@ -36,6 +36,8 @@ import renpy.sl2
 # A list of parse error messages.
 parse_errors = [ ]
 
+from renpy.parsersupport import match_logical_word
+
 
 class ParseError(Exception):
 
@@ -43,6 +45,9 @@ class ParseError(Exception):
         message = u"File \"%s\", line %d: %s" % (unicode_filename(filename), number, msg)
 
         if line:
+            if isinstance(line, list):
+                line = "".join(line)
+
             lines = line.split('\n')
 
             if len(lines) > 1:
@@ -121,6 +126,7 @@ def unicode_filename(fn):
     # Insane systems, mojibake.
     return fn.decode("latin-1")
 
+
 # Matches either a word, or something else. Most magic is taken care of
 # before this.
 lllword = re.compile(r'__(\w+)|\w+| +|.', re.S)
@@ -169,11 +175,12 @@ def unelide_filename(fn):
 
     return fn
 
+
 # The filename that the start and end positions are relative to.
 original_filename = ""
 
 
-def list_logical_lines(filename, filedata=None, linenumber=1):
+def list_logical_lines(filename, filedata=None, linenumber=1, add_lines=False):
     """
     Reads `filename`, and divides it into logical lines.
 
@@ -190,8 +197,8 @@ def list_logical_lines(filename, filedata=None, linenumber=1):
     if filedata:
         data = filedata
     else:
-        f = codecs.open(filename, "r", "utf-8")
-        data = f.read()
+        f = open(filename, "rb")
+        data = f.read().decode("utf-8")
         f.close()
 
     filename = elide_filename(filename)
@@ -209,23 +216,29 @@ def list_logical_lines(filename, filedata=None, linenumber=1):
     # The current position we're looking at in the buffer.
     pos = 0
 
+    # Are we looking at a triple-quoted string?
+
     # Skip the BOM, if any.
     if len(data) and data[0] == u'\ufeff':
         pos += 1
 
-    if renpy.game.context().init_phase:
+    if add_lines or renpy.game.context().init_phase:
         lines = renpy.scriptedit.lines
     else:
         lines = { }
 
+    len_data = len(data)
+
+    renpy.scriptedit.files.add(filename)
+
     # Looping over the lines in the file.
-    while pos < len(data):
+    while pos < len_data:
 
         # The line number of the start of this logical line.
         start_number = number
 
         # The line that we're building up.
-        line = ""
+        line = [ ]
 
         # The number of open parenthesis there are right now.
         parendepth = 0
@@ -235,16 +248,20 @@ def list_logical_lines(filename, filedata=None, linenumber=1):
 
         endpos = None
 
-        while pos < len(data):
+        while pos < len_data:
 
+            startpos = pos
             c = data[pos]
 
-            if c == '\t':
+            if c == u'\t':
                 raise ParseError(filename, number, "Tab characters are not allowed in Ren'Py scripts.")
 
-            if c == '\n' and not parendepth:
+            if c == u'\n' and not parendepth:
+
+                line = ''.join(line)
+
                 # If not blank...
-                if not re.match("^\s*$", line):
+                if not re.match(u"^\s*$", line):
 
                     # Add to the results.
                     rv.append((filename, start_number, line))
@@ -254,7 +271,7 @@ def list_logical_lines(filename, filedata=None, linenumber=1):
 
                 lines[loc].end_delim = endpos + 1
 
-                while data[endpos-1] in ' \r':
+                while data[endpos-1] in u' \r':
                     endpos -= 1
 
                 lines[loc].end = endpos
@@ -265,96 +282,112 @@ def list_logical_lines(filename, filedata=None, linenumber=1):
                 number += 1
                 endpos = None
                 # This helps out error checking.
-                line = ""
+                line = [ ]
                 break
 
-            if c == '\n':
+            if c == u'\n':
                 number += 1
                 endpos = None
 
-            if c == "\r":
+            if c == u"\r":
                 pos += 1
                 continue
 
             # Backslash/newline.
-            if c == "\\" and data[pos+1] == "\n":
+            if c == u"\\" and data[pos+1] == u"\n":
                 pos += 2
                 number += 1
-                line += "\\\n"
+                line.append(u"\\\n")
                 continue
 
             # Parenthesis.
-            if c in ('(', '[', '{'):
+            if c in u'([{':
                 parendepth += 1
 
-            if c in ('}', ']', ')') and parendepth:
+            if (c in u'}])') and parendepth:
                 parendepth -= 1
 
             # Comments.
-            if c == '#':
+            if c == u'#':
                 endpos = pos
 
-                while data[pos] != '\n':
+                while data[pos] != u'\n':
                     pos += 1
 
                 continue
 
             # Strings.
-            if c in ('"', "'", "`"):
+            if c in u'"\'`':
                 delim = c
-                line += c
+                line.append(c)
                 pos += 1
 
                 escape = False
+                triplequote = False
 
-                while pos < len(data):
+                if (pos < len_data - 1) and (data[pos] == delim) and (data[pos+1] == delim):
+                    line.append(delim)
+                    line.append(delim)
+                    pos += 2
+                    triplequote = True
+
+                while pos < len_data:
 
                     c = data[pos]
 
-                    if c == '\n':
+                    if c == u'\n':
                         number += 1
 
-                    if c == '\r':
+                    if c == u'\r':
                         pos += 1
                         continue
 
                     if escape:
                         escape = False
                         pos += 1
-                        line += c
+                        line.append(c)
                         continue
 
                     if c == delim:
-                        pos += 1
-                        line += c
-                        break
 
-                    if c == '\\':
+                        if not triplequote:
+                            pos += 1
+                            line.append(c)
+                            break
+
+                        if (pos < len_data - 2) and (data[pos+1] == delim) and (data[pos+2] == delim):
+                            pos += 3
+                            line.append(delim)
+                            line.append(delim)
+                            line.append(delim)
+                            break
+
+                    if c == u'\\':
                         escape = True
 
-                    line += c
+                    line.append(c)
                     pos += 1
 
                     continue
 
                 continue
 
-            m = lllword.match(data, pos)
+            word, magic, end = match_logical_word(data, pos)
 
-            word = m.group(0)
-            rest = m.group(1)
+            if magic:
 
-            if rest and "__" not in rest:
-                word = prefix + rest
+                rest = word[2:]
 
-            line += word
+                if u"__" not in rest:
+                    word = prefix + rest
 
-            if len(line) > 65536:
+            line.append(word)
+            pos = end
+
+            if (pos - startpos) > 65536:
                 raise ParseError(filename, start_number, "Overly long logical line. (Check strings and parenthesis.)", line=line, first=True)
 
-            pos = m.end(0)
-
-    if not line == "":
+    if line:
         raise ParseError(filename, start_number, "is not terminated with a newline. (Check strings and parenthesis.)", line=line, first=True)
 
     return rv
@@ -424,6 +457,7 @@ def group_logical_lines(lines):
 
     return gll_core(0, 0)[0]
 
+
 # A list of keywords which should not be parsed as names, because
 # there is a huge chance of confusion.
 #
@@ -690,18 +724,30 @@ class Lexer(object):
         # Strip off delimiters.
         s = s[1:-1]
 
+        def dequote(m):
+            c = m.group(1)
+
+            if c == "{":
+                return "{{"
+            elif c == "[":
+                return "[["
+            elif c == "%":
+                return "%%"
+            elif c == "n":
+                return "\n"
+            elif c[0] == 'u':
+                group2 = m.group(2)
+
+                if group2:
+                    return unichr(int(m.group(2), 16))
+            else:
+                return c
+
         if not raw:
 
             # Collapse runs of whitespace into single spaces.
             s = re.sub(r'\s+', ' ', s)
-
-            s = s.replace("\\n", "\n")
-            s = s.replace("\\{", "{{")
-            s = s.replace("\\[", "[[")
-            s = s.replace("\\%", "%%")
-            s = re.sub(r'\\u([0-9a-fA-F]{1,4})',
-                       lambda m : unichr(int(m.group(1), 16)), s)
-            s = re.sub(r'\\(.)', r'\1', s)
+            s = re.sub(r'\\(u([0-9a-fA-F]{1,4})|.)', dequote, s)
 
         return s
 
@@ -974,7 +1020,7 @@ class Lexer(object):
 
         return False
 
-    def simple_expression(self, comma=False):
+    def simple_expression(self, comma=False, operator=True):
         """
         Tries to parse a simple_expression. Returns the text if it can, or
         None if it cannot.
@@ -1020,7 +1066,7 @@ class Lexer(object):
 
                 break
 
-            if self.match(operator_regexp):
+            if operator and self.match(operator_regexp):
                 continue
 
             if comma and self.match(r','):
@@ -1043,6 +1089,12 @@ class Lexer(object):
 
         return self.simple_expression(comma=True)
 
+    def say_expression(self):
+        """
+        Parses the name portion of a say statement.
+        """
+        return self.simple_expression(operator=False)
+
     def checkpoint(self):
         """
         Returns an opaque representation of the lexer state. This can be
@@ -1077,7 +1129,7 @@ class Lexer(object):
         object, which is called directly.
         """
 
-        if isinstance(thing, str):
+        if isinstance(thing, basestring):
             name = name or thing
             rv = self.match(thing)
         else:
@@ -2115,6 +2167,7 @@ def screen2_statement(l, loc):
 
     return rv
 
+
 # The version of screen language to use by default.
 default_screen_language = int(os.environ.get("RENPY_SCREEN_LANGUAGE", "2"))
 
@@ -2169,11 +2222,12 @@ def translate_strings(init_loc, language, l):
 
     def parse_string(s):
         s = s.strip()
-        s = 'u' + s
 
         try:
-            return eval(s)
+            bc = compile(s, "<string>", "eval", renpy.python.new_compile_flags, 1)
+            return eval(bc, renpy.store.__dict__)
         except:
+            raise
             ll.error('could not parse string')
 
     while ll.advance():
@@ -2350,6 +2404,40 @@ def style_statement(l, loc):
     return rv
 
 
+def finish_say(l, loc, who, what, attributes=None):
+
+    if what is None:
+        return None
+
+    interact = True
+    with_ = None
+    arguments = None
+
+    while True:
+
+        if l.keyword('nointeract'):
+            interact = False
+
+        elif l.keyword('with'):
+            if with_ is not None:
+                l.error('say can only take a single with clause')
+
+            with_ = l.require(l.simple_expression)
+
+        else:
+            args = parse_arguments(l)
+
+            if args is None:
+                break
+
+            if arguments is not None:
+                l.error('say can only take a single set of arguments')
+
+            arguments = args
+
+    return ast.Say(loc, who, what, with_, attributes=attributes, interact=interact, arguments=arguments)
+
+
 @statement("")
 def say_statement(l, loc):
 
@@ -2358,21 +2446,20 @@ def say_statement(l, loc):
     # Try for a single-argument say statement.
     what = l.string()
 
-    if l.keyword('with'):
-        with_ = l.require(l.simple_expression)
-    else:
-        with_ = None
+    rv = finish_say(l, loc, None, what)
+
+    if (rv is not None) and l.eol():
 
-    if what is not None and l.eol():
         # We have a one-argument say statement.
         l.expect_noblock('say statement')
         l.advance()
-        return ast.Say(loc, None, what, with_)
+
+        return rv
 
     l.revert(state)
 
     # Try for a two-argument say statement.
-    who = l.simple_expression()
+    who = l.say_expression()
 
     attributes = [ ]
     while True:
@@ -2394,21 +2481,15 @@ def say_statement(l, loc):
 
     what = l.string()
 
-    if l.keyword('nointeract'):
-        interact = False
-    else:
-        interact = True
+    if (who is not None) and (what is not None):
 
-    if l.keyword('with'):
-        with_ = l.require(l.simple_expression)
-    else:
-        with_ = None
+        rv = finish_say(l, loc, who, what, attributes)
 
-    if who and what is not None:
         l.expect_eol()
         l.expect_noblock('say statement')
         l.advance()
-        return ast.Say(loc, who, what, with_, attributes=attributes, interact=interact)
+
+        return rv
 
     # This reports a parse error for any bad statement.
     l.error('expected statement.')
