@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2017 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -60,6 +60,7 @@ class Positional(object):
         if parser:
             parser.add(self)
 
+
 # Used to generate the documentation
 all_keyword_names = set()
 
@@ -213,7 +214,7 @@ class Parser(object):
             screens.
 
         `block_only`
-            If true, only parse the
+            If true, only parse the block and not the initial properties.
         """
 
         seen_keywords = set()
@@ -291,8 +292,8 @@ class Parser(object):
                 loc = l.get_location()
 
                 if l.keyword(r'has'):
-                    if self.nchildren != 1:
-                        l.error("The %s statement does not take a layout." % self.name)
+                    if not can_has:
+                        l.error("The has statement is not allowed here.")
 
                     if child_index != 0:
                         l.error("The has statement may not be given after a child has been supplied.")
@@ -373,6 +374,7 @@ class Parser(object):
 def add(thing):
     parser.add(thing)
 
+
 # A singleton value.
 many = renpy.object.Sentinel("many")
 
@@ -655,6 +657,7 @@ class IfParser(Parser):
 
         return rv
 
+
 if_statement = IfParser("if", slast.SLIf, True)
 IfParser("showif", slast.SLShowIf, False)
 
@@ -679,6 +682,7 @@ class ForParser(Parser):
             if l.match(r"\("):
                 name = self.name_or_tuple_pattern(l)
                 l.require(r'\)')
+                pattern = True
             else:
                 name = l.name()
 
@@ -729,6 +733,7 @@ class ForParser(Parser):
 
         return rv
 
+
 ForParser("for")
 
 
@@ -745,6 +750,7 @@ class OneLinePythonParser(Parser):
         code = renpy.ast.PyCode(source, loc)
         return slast.SLPython(loc, code)
 
+
 OneLinePythonParser("$")
 
 
@@ -764,6 +770,7 @@ class MultiLinePythonParser(Parser):
         code = renpy.ast.PyCode(source, loc)
         return slast.SLPython(loc, code)
 
+
 MultiLinePythonParser("python")
 
 
@@ -775,6 +782,7 @@ class PassParser(Parser):
 
         return slast.SLPass(loc)
 
+
 PassParser("pass")
 
 
@@ -791,6 +799,7 @@ class DefaultParser(Parser):
 
         return slast.SLDefault(loc, name, rest)
 
+
 DefaultParser("default")
 
 
@@ -815,7 +824,7 @@ class UseParser(Parser):
             l.expect_block("use statement")
 
             block = slast.SLBlock(loc)
-            self.parse_contents(l, block, block_only=True)
+            self.parse_contents(l, block, can_has=True, block_only=True)
 
         else:
             l.expect_eol()
@@ -825,6 +834,7 @@ class UseParser(Parser):
 
         return slast.SLUse(loc, target, args, id_expr, block)
 
+
 UseParser("use")
 Keyword("style_prefix")
 Keyword("style_group")
@@ -836,6 +846,7 @@ class TranscludeParser(Parser):
         l.expect_eol()
         return slast.SLTransclude(loc)
 
+
 TranscludeParser("transclude")
 
 
@@ -948,6 +959,7 @@ class ScreenParser(Parser):
 
         return screen
 
+
 screen_parser = ScreenParser()
 Keyword("modal")
 Keyword("zorder")
@@ -956,6 +968,7 @@ Keyword("predict")
 Keyword("style_group")
 Keyword("style_prefix")
 Keyword("layer")
+parser = None
 
 
 def init():
